import 'package:cramr_ui_lib/model/cardinality_selection.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

import 'settings_service.dart';

/// A class that many Widgets can interact with to read user settings, update
/// user settings, or listen to user settings changes.
///
/// Controllers glue Data Services to Flutter Widgets. The SettingsController
/// uses the SettingsService to store and retrieve user settings.
class SettingsController with ChangeNotifier {
  SettingsController(
      this._settingsService, this._themeMode, this._cardinalitySelection);

  // Make SettingsService a private variable so it is not used directly.
  final SettingsService _settingsService;

  // Make ThemeMode a private variable so it is not updated directly without
  // also persisting the changes with the SettingsService.
  ThemeMode _themeMode;

  // Allow Widgets to read the user's preferred ThemeMode.
  ThemeMode get themeMode => _themeMode;

  // Make CardinalitySelection a private variable so that it is not updated
  // directly without persisting changes with the SettingsService
  CardinalitySelection _cardinalitySelection;

  // Allow Widgets to read the user's preferred CardinalitySelection
  CardinalitySelection get cardinalitySelection => _cardinalitySelection;

  /// Load the user's settings from the SettingsService. It may load from a
  /// local database or the internet. The controller only knows it can load the
  /// settings from the service.
  Future<void> loadSettings() async {
    //_themeMode = await _settingsService.themeMode();
    //_cardinalitySelection = await _settingsService.cardinalitySelection();

    // Important! Inform listeners a change has occurred.
    notifyListeners();
  }

  Future<void> toggleSingularEnabled() async {
    _cardinalitySelection.toggleSingularEnabled();
    notifyListeners();
    await _settingsService.updateCardinalitySelection(_cardinalitySelection);
  }

  Future<void> togglePluralEnabled() async {
    _cardinalitySelection.togglePluralEnabled();
    notifyListeners();
    await _settingsService.updateCardinalitySelection(_cardinalitySelection);
  }

  /// Update and persist the ThemeMode based on the user's selection.
  Future<void> updateThemeMode(ThemeMode? newThemeMode) async {
    if (newThemeMode == null) return;

    // Dot not perform any work if new and old ThemeMode are identical
    //if (newThemeMode == _themeMode) return;

    // Otherwise, store the new theme mode in memory
    _themeMode = newThemeMode;

    // Important! Inform listeners a change has occurred.
    notifyListeners();

    // Persist the changes to a local database or the internet using the
    // SettingService.
    await _settingsService.updateThemeMode(newThemeMode);
  }
}
